#!/usr/bin/env perl
#
# $Id: mk_trie_regexp.pl,v 0.1 2005/09/10 20:19:44 dankogai Exp dankogai $
#
use strict;
use warnings;

package Regexp::Trie;

sub new{ bless {} => shift }
sub add{
    my $self = shift;
    my $str  = shift;
    my $ref  = $self;
    for my $char (split //, $str){
        $ref->{$char} ||= {};
        $ref = $ref->{$char};
    }
    $ref->{''} = 1; # { '' => 1 } as terminator
    $self;
}
sub _regexp{
    my $self = shift;
    return if $self->{''} and scalar keys %$self == 1; # terminator
    my (@alt, @cc);
    my $q = 0;
    for my $char (sort keys %$self){
        my $qchar = quotemeta $char;
        if (ref $self->{$char}){
            if (defined (my $recurse = _regexp($self->{$char}))){
                push @alt, $qchar . $recurse;
            }else{
                push @cc, $qchar;
            }
        }else{
            $q = 1;
        }
    }
    my $cconly = !@alt;
    @cc and push @alt, @cc == 1 ? $cc[0] : '['. join('', @cc). ']';
    my $result = @alt == 1 ? $alt[0] : '(?:' . join('|', @alt) . ')';
    $q and $result = $cconly ? "$result?" : "(?:$result)?";
    return $result;
}
sub as_regexp{ my $str = shift->_regexp; qr/$str/ }

package main;

my $src = shift || die "$0 src [dst]";
my $dst = shift || "$src.rx";
my $trie = Regexp::Trie->new;

my $count;
$|=1;
open my $in, "<:raw", $src or die "$src : $!";
while(<$in>){
    chomp;
    $trie->add($_);
    ++$count % 1000 == 0 and print "$count\r";
}
close $in;
print "$count\n";
system ("ps v$$");
my $qr = $trie->as_regexp;
open my $out, ">:raw", $dst or die "$dst : $!";
print $out 'qr{'.$qr.'}';
close $out;
system ("ps v$$");

__END__